#include <stdlib.h> // Access to 'atoll' def.

extern "C" long long int atoll(const char* string);

#include <functional>
#include <iostream>

#include "N_INSEE.h"

N_INSEE::N_INSEE(const std::string& valeur) : _valeur(0LL) {
	// Conversion from 'std::string' to 'long long'
	if (valeur.size() != 13) throw "N.INSEE requires 13 digits";
	if (!(valeur[0] == '1') && !(valeur[0] == '2')) throw "N.INSEE starts with '1' or '2'";
	_valeur = ::atoll(valeur.c_str()); // Standard 'atol' function ('const char*' -> 'long') doesn't work because 'valeur' string is too big...
}

int N_INSEE::clef() const {
	return 97 - static_cast<int>(_valeur % 97);
}

int main(int argc, char** argv) {
	// long long n_insee_de_Franck_Barbier = 1630125388055LL;
	// std::cout << "Key of " << n_insee_de_Franck_Barbier << " is equal to: " << (97LL - n_insee_de_Franck_Barbier % 97LL) << std::endl;

	N_INSEE FB("1630125388055"); // Clef -> '29'
	std::cout << "Key of 1630125388055 is equal to: " << FB.clef() << std::endl;
	// long long N_INSEE::*p_valeur = &N_INSEE::_valeur; // Compilation error: pointers on class members require 'public:'
	int (N_INSEE:: * p_clef) () const = &N_INSEE::clef;
	std::cout << "Key of 1010264445244 is equal to: " << std::invoke(p_clef, FB) << std::endl; // C++17: '#include <functional>'

	decltype(FB) JBD("1010264445244"); // Clef -> '34'
	std::cout << "Key of 1630125388055 is equal to: " << JBD.clef() << std::endl;

	return 0;
}

